/*
 *   Copyright 2005 by Aaron Seigo <aseigo@kde.org>
 *   Copyright 2008 by Andrew Lake <jamboarder@yahoo.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef PLASMA_PAINTUTILS_H
#define PLASMA_PAINTUTILS_H

#include <QtGui/QApplication>
#include <QtGui/QGraphicsItem>
#include <QtGui/QPainterPath>

#include <plasma/plasma_export.h>
#include "theme.h"

/** @headerfile plasma/paintutils.h <Plasma/PaintUtils> */

namespace Plasma
{

class Svg;

/**
 *  Namespace for all Image Effects specific to Plasma
 **/
namespace PaintUtils
{

/**
 * Creates a blurred shadow of the supplied image.
 */
PLASMA_EXPORT void shadowBlur(QImage &image, int radius, const QColor &color);

/**
 * Returns a pixmap containing text with blurred shadow.
 * Text and shadow colors default to Plasma::Theme colors.
 */
PLASMA_EXPORT QPixmap shadowText(QString text,
    const QFont &font,
    QColor textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor),
    QColor shadowColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor),
    QPoint offset = QPoint(1,1),
    int radius = 2);

PLASMA_EXPORT QPixmap shadowText(QString text,
    QColor textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor),
    QColor shadowColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor),
    QPoint offset = QPoint(1,1),
    int radius = 2);

PLASMA_EXPORT QPixmap texturedText(const QString &text, const QFont &font, Plasma::Svg *texture);

PLASMA_EXPORT void drawHalo(QPainter *painter, const QRectF &rect);

/**
 * Returns a nicely rounded rectanglular path for painting.
 */
PLASMA_EXPORT QPainterPath roundedRectangle(const QRectF &rect, qreal radius);

/**
 * center two pixmap together in the biggest rectangle
 * @since 4.5
 */
PLASMA_EXPORT void centerPixmaps(QPixmap &from, QPixmap &to);

/**
 * Blends a pixmap into another
 */
PLASMA_EXPORT QPixmap transition(const QPixmap &from, const QPixmap &to, qreal amount);

} // PaintUtils namespace

} // Plasma namespace

#endif
