/*
    Copyright 2013 Albert Vaca <albertvaka@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) version 3, or any
    later version accepted by the membership of KDE e.V. (or its
    successor approved by the membership of KDE e.V.), which shall
    act as a proxy defined in Section 6 of version 3 of the license.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library. If not, see <http://www.gnu.org/licenses/>.
*/

#include "phonepowerbattery.h"

#include <QDebug>

using namespace Solid::Backends::PhonePower;

Battery::Battery(PhonePowerDevice *device)
    : DeviceInterface(device)
    , m_chargePercent(0)
    , m_chargeState(Solid::Battery::NoCharge)
    , mDevice(device)
{
    connect(device, SIGNAL(changed()), this, SLOT(slotChanged()));
}

Battery::~Battery()
{
}

bool Battery::isPlugged() const
{
    return true;
}

Solid::Battery::BatteryType Battery::type() const
{
    return Solid::Battery::PhoneBattery;
}

int Battery::chargePercent() const
{
    m_chargePercent = mDevice->chargePercent();
    return m_chargePercent;
}

int Battery::capacity() const
{
    return 100;
}

bool Battery::isRechargeable() const
{
    return true;
}

bool Battery::isPowerSupply() const
{
    return false;
}

Solid::Battery::ChargeState Battery::chargeState() const
{
    m_chargeState = mDevice->chargeState();
    return m_chargeState;
}

void Battery::slotChanged()
{

    if (!mDevice) {
        return;
    }

    if (m_chargePercent != mDevice->chargePercent()) {
        m_chargePercent = mDevice->chargePercent();
        qDebug() << "Percentage has changed: " << m_chargePercent << mDevice->udi();
        Q_EMIT chargePercentChanged(m_chargePercent, mDevice->udi());
    }

    if (m_chargeState != mDevice->chargeState()) {
        m_chargeState = mDevice->chargeState();
        qDebug() << "State has changed: " << m_chargeState << mDevice->udi();
        Q_EMIT chargeStateChanged(m_chargeState, mDevice->udi());
    }

}

#include "backends/phonepower/phonepowerbattery.moc"
