/*
    Copyright 2013 Albert Vaca <albertvaka@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) version 3, or any
    later version accepted by the membership of KDE e.V. (or its
    successor approved by the membership of KDE e.V.), which shall
    act as a proxy defined in Section 6 of version 3 of the license.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library. If not, see <http://www.gnu.org/licenses/>.
*/

#include "phonepowerdevice.h"
#include "phonepowerdeviceinterface.h"
#include "phonepowerbattery.h"

#include <solid/genericinterface.h>
#include <solid/device.h>

#include <QtCore/QStringList>
#include <QtCore/QDebug>
#include <QtDBus/QDBusPendingReply>

using namespace Solid::Backends::PhonePower;

PhonePowerDevice::PhonePowerDevice(const QString &udi, const QString &id)
    : Solid::Ifaces::Device()
    , m_device("org.kde.kdeconnect",
               "/modules/kdeconnect/devices/"+id,
               "org.kde.kdeconnect.device",
               QDBusConnection::sessionBus())
    , m_battery("org.kde.kdeconnect",
               "/modules/kdeconnect/devices/"+id,
               "org.kde.kdeconnect.device.battery",
               QDBusConnection::sessionBus())
    , m_udi(udi)
{
    qDebug() << "/modules/kdeconnect/devices/"+id;

    if (m_battery.isValid())
        connect(&m_battery, SIGNAL(chargingChange()), this, SLOT(slotChanged()));
}

PhonePowerDevice::~PhonePowerDevice()
{
}

QObject* PhonePowerDevice::createDeviceInterface(const Solid::DeviceInterface::Type& type)
{
    if (!queryDeviceInterface(type)) {
        return 0;
    }

    DeviceInterface *iface = 0;
    switch (type)
    {
    case Solid::DeviceInterface::Battery:
        iface = new Battery(this);
        break;
    default:
        break;
    }
    return iface;
}

bool PhonePowerDevice::queryDeviceInterface(const Solid::DeviceInterface::Type& type) const
{
    switch(type) {
        case Solid::DeviceInterface::Battery:
            return true;
        default:
            return false;
    }
}

QStringList PhonePowerDevice::emblems() const
{
    return QStringList();
}

QString PhonePowerDevice::description() const
{
    return QObject::tr("%1 Battery", "%1 is battery technology").arg(batteryTechnology());
}

QString PhonePowerDevice::batteryTechnology() const
{
    return QLatin1String("Android");
}

QString PhonePowerDevice::icon() const
{
    return "pda";
}

QString PhonePowerDevice::product() const
{
    return m_device.property("name").toString();
}

QString PhonePowerDevice::vendor() const
{
    return QLatin1String("Android");
}

QString PhonePowerDevice::udi() const
{
    return m_udi;
}

QString PhonePowerDevice::parentUdi() const
{
    return QLatin1String("/org/kde/kdeconnect");
}

int PhonePowerDevice::chargePercent()
{
    return m_battery.property("charge").toInt();
}

Solid::Battery::ChargeState PhonePowerDevice::chargeState()
{
    bool charging = m_battery.property("isCharging").toBool();
    if (charging) return Solid::Battery::Charging;
    else return Solid::Battery::NoCharge;
    //TODO: return Solid::Battery::Discharging;
}

void PhonePowerDevice::slotChanged()
{
    qDebug() << "charge information changed" << m_udi;
    emit changed();
}