/*
    Copyright 2013 Albert Vaca <albertvaka@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) version 3, or any
    later version accepted by the membership of KDE e.V. (or its
    successor approved by the membership of KDE e.V.), which shall
    act as a proxy defined in Section 6 of version 3 of the license.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PHONEPOWERMANAGER_H
#define PHONEPOWERMANAGER_H

#include "solid/ifaces/devicemanager.h"

#include <QtDBus/QDBusInterface>
#include <QtCore/QSet>

#include "kdeconnectinterface.h"

namespace Solid
{
namespace Backends
{
namespace PhonePower
{

class PhonePowerDevice;

class PhonePowerManager : public Solid::Ifaces::DeviceManager
{
    Q_OBJECT

public:
    PhonePowerManager(QObject *parent);
    virtual ~PhonePowerManager();
    virtual QObject* createDevice(const QString& udi);
    virtual QStringList devicesFromQuery(const QString& parentUdi, Solid::DeviceInterface::Type type);
    virtual QStringList allDevices();
    virtual QSet< Solid::DeviceInterface::Type > supportedInterfaces() const;
    virtual QString udiPrefix() const;

private Q_SLOTS:
    void serviceReady();
    void serviceUnregistered();
    void deviceVisibilityChanged(const QString& id ,bool visible);
    void slotDeviceAdded(const QString &opath);
    void slotDeviceRemoved(const QString &opath);
    void pluginsChanged();

private:
    QSet<Solid::DeviceInterface::Type> m_supportedInterfaces;
    QSet<QString> m_devices;
    OrgKdeKdeconnectDaemonInterface m_manager;
    bool m_ready;

};

}
}
}
#endif // PHONEPOWERMANAGER_H
